--[[

ShadowSeer -- Monitors ISB uptime
By Arelenda

]]


-- ACTIVE beta v0.92


-- v0.93a
-- fixed trinket reporting

-- v0.93
-- Added Fire Spells
-- Fixed Darkmoon Crusade card, I hope (thanks Onyajon!), engaging in melee and getting melee buff stacks might confuse it, though.



-- v0.92
-- improved estimation on your gains with added spellpower
-- some minor GUI tweaks

-- fixed 0.91
-- fixed a bug where when a player not in your raid dueling another causing a shadow bolt crit would bring up an error

	
-- suggestions
-- todo: fix partial/complete absorbs
-- todo: fix immunes
-- todo: fix succubus
-- todo: customise reporting / extend it
-- record who's using charges on ISB
-- extras for comparing: spell dps, spell dpm including lifetap
-- TWEAK: improve soulshatter feedback
-- drain life/rof/hellfire are channeled, make them use the same mechanic

-- BUG: ISB assumption (not all warlocks have it) : add toggles for other warlocks : not working
-- test: detect it on self, fix hit/crit damage estimate if no ISB
-- fix: class colors for ISBless locks and Total/Last
-- improve visual feedback on "last combat" or "all combat" data flag
-- sort tables as well ?
-- monitor only warlocks/shadow priest spells, filter for name
-- add a "clipboard tooltip" option



--
-- INIT 
--
SHADOWSEER_VERSION = "v0.93a"
ShadowSeer_DataVersion = "none"

SLASH_ShadowSeer_COMMAND1 = "/shadowseer"
SLASH_ShadowSeer_COMMAND2 = "/ss"

ShadowSeer_BARHEIGHT = 14
ShadowSeer_BARWIDTH = 157
ShadowSeer_MAX_BARS = 25

ShadowSeer_Visible_Bar_Count = 10
ShadowSeer_bars = {}
ShadowSeer_text = {}

-- list of people in raid that damaged stuff
ShadowSeer_PlayerList_AllCombats = {} -- lists all combats
ShadowSeer_PlayerList_LastCombat = {} -- lists just last one

-- viewing toggles
ShadowSeer_ViewMode = "PlayerList" 
ShadowSeer_ViewingAllCombats = true

ShadowSeer_DebuffedList = {}
ShadowSeer_TargetList = {}
ShadowSeer_RecentCrits = {}

ShadowSeer_Enabled = true
ShadowSeer_Visible = true

-- target vars
ShadowSeer_MySpell = nil
ShadowSeer_MyRank = nil
ShadowSeer_MyTarget = nil
ShadowSeer_ActiveBuffList = nil
ShadowSeer_TimeCastStarted = 0
ShadowSeer_SpellPowerAtStart = 0


-- delayed scan for buffs (since we get a "you received BUFF" event before we have the buff)
ShadowSeer_BuffsToScan = {}
-- (name, received)

ShadowSeer_WarlockNamesWithoutISB = {} -- list of warlocks without ISB

-- GUI
ShadowSeer_Selected_Index = -1
ShadowSeer_Tooltip_Visible = false
ShadowSeer_Tooltip_Text = nil
ShadowSeer_Tooltip_Index = nil
ShadowSeer_ReportMode = "PARTY" -- "PARTY", "RAID", "SAY", "WHISPER"
ShadowSeer_ReportWhisperTarget = nil

-- key bindings
BINDING_HEADER_SHADOWSEER = "Shadowseer"
BINDING_NAME_SHADOWSEERRAIDDATA = "Show Raid Data"
BINDING_NAME_SHADOWSEERSPELLDATA = "Show Spell Data"
BINDING_NAME_SHADOWSEERPROCDATA = "Show Procs/Trinket Data"
BINDING_NAME_SHADOWSEERTOGGLECOMBAT = "Toggle Last Combat/All combats"
BINDING_NAME_SHADOWSEERSCROLLUP = "Scroll Tooltip Cursor up"
BINDING_NAME_SHADOWSEERSCROLLDOWN = "Scroll Tooltip Cursor down"
BINDING_NAME_SHADOWSEERREPORTLINE = "Report Tooltip Line"

-- log vars
ShadowSeer_LogString = ""
ShadowSeer_Time_Start = 0
ShadowSeer_Time_Last_Combat_Start = 0
ShadowSeer_LoggingEnabled = false
ShadowSeer_LogLastWasDebuff = false


--
-- CUSTOMIZATION
--

-- chat text color
SHADOWSEER_TEXT_COLOR_RED = 0
SHADOWSEER_TEXT_COLOR_GREEN = 0.8
SHADOWSEER_TEXT_COLOR_BLUE = 0.8

-- frame color
SHADOWSEER_BGCOLOR_RED = 0.4
SHADOWSEER_BGCOLOR_GREEN = 0.4
SHADOWSEER_BGCOLOR_BLUE = 0.7
SHADOWSEER_BGCOLOR_ALPHA = 0.9

--
-- DEBUG code
--
function ErrorWithStack(msg)
   msg = msg.."\n"..debugstack()
   	DEFAULT_CHAT_FRAME:AddMessage("ShadowSeer: ERROR : " .. msg, SHADOWSEER_TEXT_COLOR_RED, SHADOWSEER_TEXT_COLOR_GREEN, SHADOWSEER_TEXT_COLOR_BLUE)
   _ERRORMESSAGE(msg)
end

function ShadowSeer_Log(msg, a1, a2, a3, a4)
	if (not ShadowSeer_LoggingEnabled) then
		return
	end


	if (msg == "Debuff not confirmed Yet") then
		if (ShadowSeer_LogLastWasDebuff) then
			return -- quick and dirty fix to not spam the Log
		else
			ShadowSeer_LogLastWasDebuff = true
		end
	else
		ShadowSeer_LogLastWasDebuff = false
	end

	
	if (string.len(ShadowSeer_LogString) > 250000) then
		local sindex = string.find(ShadowSeer_LogString, "Combat started", 225000)
		if (sindex) then
			sindex = string.find(ShadowSeer_LogString, "xxx", sindex) -- crop at a start of a combat if it's too long
			ShadowSeer_LogString = string.sub(ShadowSeer_LogString, sindex+3, -1)
		end
	end
	
	ShadowSeer_LogString = ShadowSeer_LogString .. GetTime()*1000 - ShadowSeer_Time_Start .. "\t"  .. GetTime()*1000 - ShadowSeer_Time_Last_Combat_Start .. "\t".. msg
	if (a1) then
		ShadowSeer_LogString = ShadowSeer_LogString .. "\t" .. a1
	end
	if (a2) then
		ShadowSeer_LogString = ShadowSeer_LogString .. "\t" .. a2
	end
	if (a3) then
		ShadowSeer_LogString = ShadowSeer_LogString .. "\t" .. a3
	end
	if (a4) then
		ShadowSeer_LogString = ShadowSeer_LogString .. "\t" .. a4
	end
	ShadowSeer_LogString = ShadowSeer_LogString .. "xxx"
end


--
-- ON LOAD
-- 
function ShadowSeer_OnLoad()
    this:RegisterEvent("VARIABLES_LOADED")

	this:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_CREATURE_DAMAGE") -- dots & shadow vuln
	this:RegisterEvent("CHAT_MSG_SPELL_PARTY_DAMAGE") -- in party damage Spells
	this:RegisterEvent("CHAT_MSG_SPELL_FRIENDLYPLAYER_DAMAGE") -- nonparty damage Spells
	this:RegisterEvent("CHAT_MSG_SPELL_SELF_DAMAGE")
	this:RegisterEvent("CHAT_MSG_COMBAT_HOSTILE_DEATH")
	this:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_SELF_DAMAGE") -- hellfire damage to self

	this:RegisterEvent("CHAT_MSG_SPELL_AURA_GONE_OTHER") -- shadow vuln fades
	this:RegisterEvent("PLAYER_TARGET_CHANGED") -- changed target, using this to log target levels

	this:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_SELF_BUFFS") -- monitor temp buffs like trinkets
	this:RegisterEvent("CHAT_MSG_SPELL_AURA_GONE_SELF") -- monitor temp buffs like trinkets
	
	this:RegisterEvent("UNIT_SPELLCAST_SENT") -- fired when sending a spell. mentions target
	this:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED") -- fired when spellcast worked. no target mentioned, so I keep track of the UNIT_SPELLCAST_SENT event target

	this:RegisterEvent("PLAYER_REGEN_DISABLED") -- entering combat
	this:RegisterEvent("PLAYER_REGEN_ENABLED") -- entering combat

	
    SlashCmdList["ShadowSeer_COMMAND"] = ShadowSeer_Command

	ShadowSeer_Initialise_Frame()
	
	ShadowSeer_Tooltip:SetOwner(ShadowSeerFrame)
	ShadowSeer_Tooltip:SetScale(this:GetScale())

	ShadowSeer_ClearLists()

-- error handler
	seterrorhandler(ErrorWithStack)
	
	ShadowSeer_ShowMessage("ShadowSeer ".. SHADOWSEER_VERSION .. " loaded. /shadowseer help")
	ShadowSeer_Log("Variables Loaded")

-- disable logging on every release version
	ShadowSeer_LoggingEnabled = true
end

--
-- COMMUNICATION FUNCTIONS
--
function ShadowSeer_ShowMessage(msg)
	DEFAULT_CHAT_FRAME:AddMessage("ShadowSeer: " .. msg, SHADOWSEER_TEXT_COLOR_RED, SHADOWSEER_TEXT_COLOR_GREEN, SHADOWSEER_TEXT_COLOR_BLUE)
end

function ShadowSeer_DebugMessage(msg)
	if (SHADOWSEER_DEBUG) then
		DEFAULT_CHAT_FRAME:AddMessage("ShadowSeer - debug: " .. msg, SHADOWSEER_TEXT_COLOR_RED, SHADOWSEER_TEXT_COLOR_GREEN, SHADOWSEER_TEXT_COLOR_BLUE)
	end
end


--
-- HELP FUNCTION
--
function ShadowSeer_ExplainOptions()
	ShadowSeer_ShowMessage("---- ShadowSeer " .. SHADOWSEER_VERSION .. " usage:")
	ShadowSeer_ShowMessage("---- /ss on")
	ShadowSeer_ShowMessage("---- /ss off")
	ShadowSeer_ShowMessage("---- /ss hide")
	ShadowSeer_ShowMessage("---- /ss show")
	ShadowSeer_ShowMessage("---- /ss help")
	ShadowSeer_ShowMessage("---- /ss clear  :: clears the list")
	ShadowSeer_ShowMessage("---- /ss whisper <name>  :: sets the whisper target for reporting")
	
	ShadowSeer_ShowMessage("---- To move the window, click the empty area and drag it elsewhere")
	if (ShadowSeer_Enabled) then
		ShadowSeer_ShowMessage("---- ShadowSeer " .. SHADOWSEER_VERSION .. " is currently enabled.")
	else
		ShadowSeer_ShowMessage("---- ShadowSeer " .. SHADOWSEER_VERSION .. " is currently disabled.")
	end
end	

-- 
-- RESPONSE TO COMMANDS
--
function ShadowSeer_Command(msg)
	msg = strlower(msg)

	if (msg == "enable" or msg == "on") then
		ShadowSeer_Enabled = true
		ShadowSeer_Visible = true
		ShadowSeerFrame:Show()
		ShadowSeer_ShowMessage("ShadowSeer enabled")
	elseif (msg == "disable" or msg == "off") then
		ShadowSeer_Enabled = false
		ShadowSeer_Visible = false
		ShadowSeerFrame:Hide()
		ShadowSeer_ShowMessage("ShadowSeer has been disabled. Typing '/ss on' will enable it again.")
	elseif (msg == "hide") then
		ShadowSeerFrame:Hide()
		ShadowSeer_Visible = false
		ShadowSeer_ShowMessage("ShadowSeer is hidden but still working in background. Typing '/ss show' will show it again.")
	elseif (msg == "show") then
		ShadowSeerFrame:Show()
		ShadowSeer_Visible = true
	elseif (msg == "log") then
		if (ShadowSeer_LoggingEnabled) then
			ShadowSeer_Command("log off")
		else
			ShadowSeer_Command("log on")
		end
	elseif (msg == "log off") then
		ShadowSeer_Log("Logging disabled by user.")	
		ShadowSeer_LoggingEnabled = false
		ShadowSeer_ShowMessage("ShadowSeer Logging Disabled.")
	elseif (msg == "log on") then
		ShadowSeer_LoggingEnabled = true
		ShadowSeer_ShowMessage("ShadowSeer Logging Enabled.")
		ShadowSeer_Log("Logging enabled by user.")	
	elseif (msg == "clear") then
		ShadowSeer_ClearLists()
		ShadowSeer_RedoBars()
		ShadowSeer_ShowMessage("ShadowSeer Damage List Cleared.")
	elseif (msg == "toggle") then
		ShadowSeer_ViewingAllCombats = not ShadowSeer_ViewingAllCombats
		ShadowSeer_RedoBars()
		if (ShadowSeer_ViewingAllCombats) then
			ShadowSeer_ShowMessage("Now Viewing data for all combats.")
		else
			ShadowSeer_ShowMessage("Now Viewing data for last combat only.")
		end
	elseif (msg == "rangeup") then
		-- up the combat log range
		SetCVar("CombatDeathLogRange" , "200")
		SetCVar("CombatLogRangeCreature" , "200")
		SetCVar("CombatLogRangeFriendlyPlayers" , "200")
		SetCVar("CombatLogRangeFriendlyPlayersPets" , "200")
		SetCVar("CombatLogRangeHostilePlayers" , "200")
		SetCVar("CombatLogRangeHostilePlayersPets" , "200")
		SetCVar("CombatLogRangeParty" , "200")
		SetCVar("CombatLogRangePartyPet" , "200")
		SetCVar("CombatModeMaxDistance" , "70.0f")
	elseif (string.sub(msg, 1, 8) == "whisper " or string.sub(msg, 1, 2) == "w ") then
		local smsg
		if (string.sub(msg, 1, 8) == "whisper ") then
			smsg = string.sub(msg, 9)
		else
			smsg = string.sub(msg, 3)
		end
		
		ShadowSeer_WhisperTarget = string.upper(string.sub(smsg, 1, 1)) .. string.lower(string.sub(smsg, 2))
		ShadowSeer_ShowMessage("Whisper target set to " .. ShadowSeer_WhisperTarget)
	else
		ShadowSeer_ExplainOptions()
	end
end

--
-- HANDLING INCOMING EVENTS (COMBAT LOG EVENTS)
--
function ShadowSeer_OnEvent(event) 
	
	if (event == "VARIABLES_LOADED") then
		ShadowSeer_ShowMessage("Welcome to ShadowSeer " .. SHADOWSEER_VERSION)
		ShadowSeer_Visible_Bar_Count = table.getn(ShadowSeer_GetActiveList())
		
		local pl = ShadowSeer_GetPlayer(ShadowSeer_PlayerList_AllCombats, UnitName("player"))
		if (pl) then
			for i = 1, table.getn(pl.proctable) do 
				pl.proctable[i].lastprocced = -1
			end
		else
			ShadowSeer_CreatePlayer(ShadowSeer_PlayerList_AllCombats, UnitName("player"), UnitClass("player"))
		end

		pl = ShadowSeer_GetPlayer(ShadowSeer_PlayerList_LastCombat, UnitName("player"))
		if (pl) then
			for i = 1, table.getn(pl.proctable) do 
				pl.proctable[i].lastprocced = -1
			end
		else
			ShadowSeer_CreatePlayer(ShadowSeer_PlayerList_LastCombat, UnitName("player"), UnitClass("player"))
		end
		
		if (UnitClass("player") == "Warlock" or UnitClass("player") == "Priest") then
			ShadowSeer_Command("on")
		else
			ShadowSeer_Command("off")
		end

		if (not ShadowSeer_ViewMode) then
			ShadowSeer_ViewMode = "PlayerList"
		end

		if (ShadowSeer_DataVersion ~= SHADOWSEER_VERSION) then
			ShadowSeer_ShowMessage("ShadowSeer has detected you updated to " .. SHADOWSEER_VERSION .. ", clearing old data.")
			ShadowSeer_ClearLists()
		end
		
		ShadowSeer_RedoBars()
		
		ShadowSeer_LogString = ""
		ShadowSeer_Time_Start = GetTime()*1000

		ShadowSeer_Log("Log Started")
		return
	end

	if (not ShadowSeer_Enabled) then
		return
	end

	if (table.getn(ShadowSeer_BuffsToScan) > 0) then
		ShadowSeer_ScanBuffs()
	end
	
	-- hostile deaths 
	if (event == "CHAT_MSG_COMBAT_HOSTILE_DEATH") then
		-- if something died, remove it from the debuffed list
		start, iend, name = string.find(arg1, "(.+) dies")
		if (start) then
			ShadowSeer_Log(arg1, target)
			ShadowSeer_RemoveFromDebuffedList(name)
		end

	-- dots
	elseif (event == "CHAT_MSG_SPELL_PERIODIC_CREATURE_DAMAGE") then
		ShadowSeer_ParseDotDamage()
		ShadowSeer_RedoBars()

	-- my spells
	elseif (event == "CHAT_MSG_SPELL_SELF_DAMAGE") then 
		ShadowSeer_ParseSelfDamage()
		ShadowSeer_RedoBars()

	-- damage spells by others
	elseif (event == "CHAT_MSG_SPELL_FRIENDLYPLAYER_DAMAGE" or event == "CHAT_MSG_SPELL_PARTY_DAMAGE") then -- friendly spells ?
		ShadowSeer_ParseOtherDamage()
		ShadowSeer_RedoBars()

	-- shadow vuln fading
	elseif (event == "CHAT_MSG_SPELL_AURA_GONE_OTHER") then 
		start, iend, debuff, target = string.find(arg1, "(.+) fades from (.+)%.")
		
		if (debuff == "Shadow Vulnerability") then
			ShadowSeer_Log(arg1, target, debuff)
			ShadowSeer_RemoveFromDebuffedList(target)
		end

	-- target changed
	elseif (event == "PLAYER_TARGET_CHANGED") then 
		if (UnitName("target") and not UnitIsFriend("target", "player")) then	
			ShadowSeer_AddToTargetList(UnitName("target"), UnitLevel("target"))
		end
	
	-- monitor own spellcasts to count our spells
	elseif (event == "UNIT_SPELLCAST_SENT") then 
		if (arg1 == "player" and ShadowSeer_IsWarlockCombatSpell(arg2)) then
			ShadowSeer_MySpell = arg2
			ShadowSeer_MyRank = arg3
			ShadowSeer_MyTarget = arg4
			if (arg4 == "") then
				ShadowSeer_MyTarget = nil
			end
			ShadowSeer_ActiveBuffList = ShadowSeer_GetActiveBuffs()
			ShadowSeer_TimeCastStarted = GetTime()

			local school = ShadowSeer_GetSchoolForSpell(arg2)
			if (school == "Shadow") then
				ShadowSeer_SpellPowerAtStart = GetSpellBonusDamage(6) 
			elseif (school == "Fire") then
				ShadowSeer_SpellPowerAtStart = GetSpellBonusDamage(3)
			else
				ShadowSeer_SpellPowerAtStart = 0
			end
			
			--ShadowSeer_ShowMessage("Started cast of " .. arg2 .. " on " .. arg4 .. " of school: " .. school .. " power: " .. ShadowSeer_SpellPowerAtStart)
			ShadowSeer_Log("Started cast of " .. arg2, arg4, "Combat Spell")
		end
	elseif (event == "UNIT_SPELLCAST_SUCCEEDED") then 
		if (arg1 == "player" and ShadowSeer_MySpell == arg2 and ShadowSeer_MyRank == arg3) then
			if (ShadowSeer_IsWarlockCombatSpell(arg2)) then -- filter out noise
				if (ShadowSeer_IsDelayedDamageSpell(arg2)) then	
					-- Direct damage spells can fail to hit, due to target dying between succesful cast and bolt landing.
					-- So we only register those when we get landing results
				elseif (arg2 == "Soulshatter") then
					-- might want to keep track of soulshatters cast here
				else
					ShadowSeer_RegisterSpellCast(ShadowSeer_MyTarget, arg2) -- for resistances
				end
				
				if (ShadowSeer_IsSpellBoostedBySpellPower(arg2)) then
					local si_all = ShadowSeer_GetPlayer(ShadowSeer_PlayerList_AllCombats).spellinfo
					local si_last = ShadowSeer_GetPlayer(ShadowSeer_PlayerList_LastCombat).spellinfo
					local school = ShadowSeer_GetSchoolForSpell(arg2)

					si_all.spellcount = si_all.spellcount + 1
					si_all.spellpower = si_all.spellpower + ShadowSeer_SpellPowerAtStart
					si_last.spellcount = si_last.spellcount + 1
					si_last.spellpower = si_last.spellpower + ShadowSeer_SpellPowerAtStart

					if (school == "Shadow") then
						si_all.shadowcount = si_all.shadowcount + 1
						si_all.shadowpower = si_all.shadowpower + ShadowSeer_SpellPowerAtStart
						si_last.shadowcount = si_last.shadowcount + 1
						si_last.shadowpower = si_last.shadowpower + ShadowSeer_SpellPowerAtStart
					elseif (school == "Fire") then 
						si_all.firecount = si_all.firecount + 1
						si_all.firepower = si_all.firepower + ShadowSeer_SpellPowerAtStart
						si_last.firecount = si_last.firecount + 1
						si_last.firepower = si_last.firepower + ShadowSeer_SpellPowerAtStart
					else
						ShadowSeer_Log("Error: can't find school for spell " .. arg2)
					end
					ShadowSeer_RegisterSpellCastInTempBuffList(arg2, ShadowSeer_ActiveBuffList) -- for trinkets
				end

				ShadowSeer_RedoBars()
--				ShadowSeer_Log("Completed cast of " .. arg2, ShadowSeer_MyTarget, "Combat Spell")
			else
--				ShadowSeer_Log("Completed cast of " .. arg2, ShadowSeer_MyTarget, "Non combat Spell")
			end
		end

	elseif (event == "CHAT_MSG_SPELL_PERIODIC_SELF_DAMAGE") then -- hellfire damage to self
		start, iend, damage = string.find(arg1, "You suffer (.+) damage from your Hellfire%.")
		if (not start) then 
			return
		end
		local sf = ShadowSeer_FindSpellField(ShadowSeer_PlayerList_AllCombats, nil, "Hellfire")
		if (sf.selfticcount) then
			sf.selfticcount = sf.selfticcount + 1
		else
			sf["selfticcount"] = 1
		end
		sf = ShadowSeer_FindSpellField(ShadowSeer_PlayerList_LastCombat, nil, "Hellfire")
		if (sf.selfticcount) then
			sf.selfticcount = sf.selfticcount + 1
		else
			sf["selfticcount"] = 1
		end
	
	elseif (event == "CHAT_MSG_SPELL_PERIODIC_SELF_BUFFS") then -- monitor trinkets  
		start, iend, buff = string.find(arg1, "You gain (.+)%.")
		if (not buff) then 
			return
		end
		
		if (string.find(buff, "health") or string.find(buff, "mana")) then
			return
		end
		-- we don't actually have the buff yet, so just scan for on next events
		for i = 1, table.getn(ShadowSeer_BuffsToScan) do
			if (ShadowSeer_BuffsToScan[i].name == buff) then
				ShadowSeer_BuffsToScan[i].received = GetTime()
				return
			end
		end
		local brec = {}
		brec["name"] = buff
		brec["received"] = GetTime()
		table.insert(ShadowSeer_BuffsToScan, brec)

	elseif (event == "CHAT_MSG_SPELL_AURA_GONE_SELF") then
		start, iend, buff = string.find(arg1, "(.+) fades from you.")
		if (buff) then
			ShadowSeer_DeactivateBuff(buff)
		end
		
	elseif (event == "PLAYER_REGEN_DISABLED") then -- entering combat
		ShadowSeer_ClearLastCombatList()
		ShadowSeer_Time_Last_Combat_Start = GetTime()*1000
		ShadowSeer_Log("Combat started")
		ShadowSeer_RedoBars()
	
	elseif (event == "PLAYER_REGEN_ENABLED") then  -- leaving combat. does nothing for now
		ShadowSeer_Log("Combat ended")
		
	-- unhandled
	else
		ShadowSeer_ShowMessage("unhandled event: " .. event)
		if (arg1) then
			ShadowSeer_ShowMessage("unhandled event arg: " .. arg1)
			ShadowSeer_Log("unhandled event arg: " .. arg1)
		end
	end
end

--
-- RESIST LIST FUNCTIONS (doesn't include partial resists)
--
function ShadowSeer_RegisterSpellCast(target, spell)
-- only monitor resists from self, due to not being able to tell other spellcasts reliably
--	ShadowSeer_ShowMessage("Registering cast of " .. spell .. " on " .. target .. " (" .. ShadowSeer_GetLevel(target) ..")")
	local sf = ShadowSeer_FindSpellField(ShadowSeer_PlayerList_AllCombats, target, spell)
	sf.castcount = sf.castcount + 1
	sf = ShadowSeer_FindSpellField(ShadowSeer_PlayerList_LastCombat, target, spell)
	sf.castcount = sf.castcount + 1
end

function ShadowSeer_RegisterResist(target, spell)
--	ShadowSeer_ShowMessage("Registering resist of " .. spell .. " on " .. target .. " (" .. ShadowSeer_GetLevel(target) ..")")
	sf = ShadowSeer_FindSpellField(ShadowSeer_PlayerList_AllCombats, target, spell)
	sf.resistcount = sf.resistcount + 1
	sf = ShadowSeer_FindSpellField(ShadowSeer_PlayerList_LastCombat, target, spell)
	sf.resistcount = sf.resistcount + 1
end
	
function ShadowSeer_FindSpellField(plist, target, spell)
	pl = ShadowSeer_GetPlayer(plist, UnitName("player"))
	targetlevel = ShadowSeer_GetLevel(target)
	rl = pl.resulttable

	for i = 1, table.getn(rl) do
		if (rl[i].spellname == spell) then
			-- found spell and level. 
			for j = 1, table.getn(rl[i].resultlist) do 
				if (rl[i].resultlist[j].level == targetlevel) then
					return rl[i].resultlist[j]
				end
			end
			-- found spell but no level field. create.
			local ShadowSeer_rec = {}
			ShadowSeer_rec["level"] = targetlevel
			ShadowSeer_rec["resistcount"] = 0
			ShadowSeer_rec["castcount"] = 0
			ShadowSeer_rec["critcount"] = 0
			ShadowSeer_rec["totaldamage"] = 0
			table.insert(rl[i].resultlist, ShadowSeer_rec)
			return ShadowSeer_rec
		end
	end
	
	-- spell not found, add it.
	local ShadowSeer_rec = {}
	ShadowSeer_rec["spellname"] = spell
	ShadowSeer_rec["resultlist"] = {}

	local Ss2_rec = {}
	Ss2_rec["level"] = targetlevel
	Ss2_rec["resistcount"] = 0
	Ss2_rec["castcount"] = 0
	Ss2_rec["critcount"] = 0
	Ss2_rec["totaldamage"] = 0
	
	table.insert(ShadowSeer_rec.resultlist, Ss2_rec)
	table.insert(rl, ShadowSeer_rec)
	return Ss2_rec
end

function ShadowSeer_RegisterOwnDamage(plist, target, spell, amount)
	local sf = ShadowSeer_FindSpellField(plist, target, spell)
	sf.totaldamage = sf.totaldamage + amount
end	

--
-- PLAYER LIST FUNCTIONS
--
function ShadowSeer_CreateDamageList(name)
	local ShadowSeer_rec = {}
	ShadowSeer_rec["spellname"] = name
	ShadowSeer_rec["critcount"] = 0
	ShadowSeer_rec["ticcount"] = 0
	ShadowSeer_rec["totaldamage"] = 0
	ShadowSeer_rec["maxdamage"] = 0
	ShadowSeer_rec["ampeddamage"] = 0
	ShadowSeer_rec["totalcritdamage"] = 0
	ShadowSeer_rec["partialresist"] = 0
	return ShadowSeer_rec
end

function ShadowSeer_CreatePlayer(plist, name, class)
	local prec = {}
	prec["name"] = name
	prec["class"] = class
	prec["damagelist"] = {}
	tinsert(prec.damagelist, ShadowSeer_CreateDamageList("All"))
	
	prec["ISBcontribution"] = 0
	prec["SBcrits"] = 0

	if (name == UnitName("player")) then
		prec["resulttable"] = {}
		prec["proctable"] = {}
		
		local p2 = {}
		p2["spellcount"] = 0
		p2["spellpower"] = 0
		p2["firecount"] = 0
		p2["shadowcount"] = 0
		p2["firepower"] = 0
		p2["shadowpower"] = 0
		
		prec["spellinfo"] = p2
	end


	table.insert(plist, prec)
end

function ShadowSeer_ClearLists()
	ShadowSeer_PlayerList_AllCombats = {}
	ShadowSeer_CreatePlayer(ShadowSeer_PlayerList_AllCombats, "All combats", "Paladin")
	ShadowSeer_CreatePlayer(ShadowSeer_PlayerList_AllCombats, UnitName("player"), UnitClass("player"))

	ShadowSeer_ClearLastCombatList()
	ShadowSeer_DebuffedList = {}
	ShadowSeer_TargetList = {}
	ShadowSeer_RecentCrits = {}
	ShadowSeer_DataVersion = SHADOWSEER_VERSION
end

function ShadowSeer_ClearLastCombatList()
	ShadowSeer_PlayerList_LastCombat = {}
	ShadowSeer_CreatePlayer(ShadowSeer_PlayerList_LastCombat, "Last combat", "Paladin")
	ShadowSeer_CreatePlayer(ShadowSeer_PlayerList_LastCombat, UnitName("player"), UnitClass("player"))
end

function ShadowSeer_GetActiveList()
	if (ShadowSeer_ViewingAllCombats) then
		return ShadowSeer_PlayerList_AllCombats
	else
		return ShadowSeer_PlayerList_LastCombat
	end
end

function ShadowSeer_GetPlayer(plist, name)
	if (not name) then
		name = UnitName("player")
	end
	
	for i=1, table.getn(plist) do
		if (plist[i].name == name) then
			return plist[i]
		end
	end
	return nil
end

function ShadowSeer_GetActivePlayer()
	return ShadowSeer_GetPlayer(ShadowSeer_GetActiveList(), UnitName("player"))
end

--
-- DEBUFF LIST / CRIT HISTORY
--
-- since there is a lag between crit and debuff application, we can not monitor whose debuff did what if it gets overwritten. 
function ShadowSeer_AddToDebuffList(target, caster)
	for i=1, table.getn(ShadowSeer_DebuffedList) do
		if (ShadowSeer_DebuffedList[i].target == target) then
			-- refresh debuff
			ShadowSeer_DebuffedList[i].time = GetTime()
			ShadowSeer_DebuffedList[i].caster = caster
			return
		end
	end

	local drec = {}
	drec["target"] = target
	drec["time"] = GetTime()
	drec["caster"] = caster

	tinsert(ShadowSeer_DebuffedList, drec)
end

function ShadowSeer_RemoveFromDebuffedList(target)
	for i=1, table.getn(ShadowSeer_DebuffedList) do
		if (ShadowSeer_DebuffedList[i].target == target) then
			table.remove(ShadowSeer_DebuffedList, i)
			return
		end
	end
end

function ShadowSeer_TargetIsDebuffed(target)
	for i=1, table.getn(ShadowSeer_DebuffedList) do
		if (ShadowSeer_DebuffedList[i].target == target) then
			return ShadowSeer_DebuffedList[i].caster
		end
	end
	return nil
end	

-- confirms that a target was recently hit by a shadow bolt
function ShadowSeer_ConfirmDebuff(target)
	-- filter stale crits 
	local i = 1
	
	while (i <= table.getn(ShadowSeer_RecentCrits)) do
		if (GetTime() - ShadowSeer_RecentCrits[i].time > 3) then -- timed out 
			table.remove(ShadowSeer_RecentCrits, i)
		else
			i = i + 1
		end
	end

	-- find match
	for i=1, table.getn(ShadowSeer_RecentCrits) do
		if (ShadowSeer_RecentCrits[i].target == target) then
			ShadowSeer_AddToDebuffList(target, ShadowSeer_RecentCrits[i].caster)
			ShadowSeer_Log("SV caused by recent Shadow Bolt Crit", target, "Shadow Vulnerability", ShadowSeer_RecentCrits[i].caster)
			return ShadowSeer_RecentCrits[i].caster
		end
	end
	
	ShadowSeer_Log("Tried to confirm SV: fail, no SB crit detected, assuming shadow priest did it", target, "Shadow Vulnerability")
	return nil
end

function ShadowSeer_AddToCritList(target, caster)
	local crec = {}
	crec["target"] = target
	crec["caster"] = caster
	crec["time"] = GetTime()
	table.insert(ShadowSeer_RecentCrits, crec)
	ShadowSeer_Log("Crit registered.", target, caster)
end

function ShadowSeer_ChangeDebuffOwner(target, caster)
	for i=1, table.getn(ShadowSeer_DebuffedList) do
		if (ShadowSeer_DebuffedList[i].target == target) then
			ShadowSeer_DebuffedList[i].caster = caster
			ShadowSeer_DebuffedList[i].time = GetTime()
		end
	end
end	

--
-- TARGET LIST
--
-- List of targeted units (to determine target levels)
function ShadowSeer_AddToTargetList(name, lvl)
	local level = lvl
	if (level == -1) then
		level = "BOSS"
	end
	
	for i=1, table.getn(ShadowSeer_TargetList) do
		if (ShadowSeer_TargetList[i].name == name) then
			ShadowSeer_TargetList[i].level = level -- already in there, just update level
			return 
		end
	end
	
	local newrec = {}
	newrec["name"] = name
	newrec["level"] = level
	table.insert(ShadowSeer_TargetList, newrec)
end

function ShadowSeer_GetLevel(name)
	if (not name) then
		return nil
	end
	for i=1, table.getn(ShadowSeer_TargetList) do
		if (ShadowSeer_TargetList[i].name == name) then
			return ShadowSeer_TargetList[i].level
		end
	end

	-- not in the list
	return "UNKNOWN"
end


--
-- ISB FLAGGING 
--
function ShadowSeer_MarkAsWithoutISB(name)
	for i = 1, table.getn(ShadowSeer_WarlockNamesWithoutISB) do 
		if (ShadowSeer_WarlockNamesWithoutISB[i] == name) then
			return
		end
	end
	table.insert(ShadowSeer_WarlockNamesWithoutISB, name)
end

function ShadowSeer_MarkAsWithISB(name)
	for i = 1, table.getn(ShadowSeer_WarlockNamesWithoutISB) do 
		if (ShadowSeer_WarlockNamesWithoutISB[i] == name) then
			table.remove(ShadowSeer_WarlockNamesWithoutISB, i)
		end
	end
end

function ShadowSeer_WarlockHasISB(name)
	if (UnitName("player") == name) then 
		return ShadowSeer_TalentsIn("Improved Shadow Bolt") == 5
	end

	for i = 1, table.getn(ShadowSeer_WarlockNamesWithoutISB) do 
		if (ShadowSeer_WarlockNamesWithoutISB[i] == name) then
			return false
		end
	end
	
	return true
end



