--
-- COMBAT LOGGING & PARSING
--
-- PARSING
function ShadowSeer_ParseDotDamage()
	-- (name) suffers (amount) (type) damage from (caster)'s (spell).
	local start, iend, resist, amount, school, spell, partialresist
	local text = arg1
		
	start, iend, partialresist = strfind(text,"%((.+) resisted%)")
	if (start) then
		text = string.sub(text, 1, string.find(text, "%."))
	else
		partialresist = 0
	end
		
	start, iend, target, amount, school, spell = string.find(text, "(.+) suffers (.+) (.+) damage from your (.+)%.")
	if (start and school == "Shadow" or school == "Fire") then
		ShadowSeer_Log(arg1, target, amount, UnitName("player"), spell)
		ShadowSeer_RegisterDoTDamage(target, amount, UnitName("player"), spell, partialresist)
		return
	end
		
	start, iend, target, amount, school, player, spell = string.find(text, "(.+) suffers (.+) (.+) damage from (.+)'s (.+)%.")
	if (start and school == "Shadow") then -- don't care about other fire users
		player = string.sub(player, 1, string.len(player)-1)

		ShadowSeer_Log(arg1, target, amount, player, spell)
		ShadowSeer_RegisterDoTDamage(target, amount, player, spell, partialresist)
		return
	end
		
	start, iend, spell, target = string.find(text, "Your (.+) was resisted by (.+)%.")
	if (start) then
		ShadowSeer_Log(arg1, target, 0, UnitName("player"), spell)
		ShadowSeer_RegisterResist(target, spell)
		return
	end
		
	start, iend, target, debuff = string.find(text, "(.+) is afflicted by (.+)%.")
	if (start and debuff == "Shadow Vulnerability") then
		ShadowSeer_Log(arg1, target, debuff)
		ShadowSeer_ConfirmDebuff(target)
		return
	elseif (start and debuff == "Soulshatter") then
		ShadowSeer_RegisterSpellCast(target, debuff)
	end
end

function ShadowSeer_ParseSelfDamage()
-- Your (spell) hit|crits (target) for (x) damage. 
-- Your (spell) hit|crits (target) for (x) damage. (y resisted)
	local start, iend, resist, amount, school, spell
	local text = arg1
		
	start, iend, partialresist = strfind(text, "%((.+) resisted%)")
	if (start) then
		text = string.sub(text, 1, string.find(text, "%."))
	else
		partialresist = 0
	end 

	start, iend, spell, target, amount = string.find(text, "Your (.+) crits (.+) for (.+) Shadow damage%.")
	if (not start) then
		start, iend, spell, target, amount = string.find(text, "Your (.+) crits (.+) for (.+) Fire damage%.")
	end

	if (start) then
		ShadowSeer_Log(arg1, target, amount, UnitName("player"), spell)

		-- special case: soC & SB
		if (spell == "Seed of Corruption") then
			spell = "Seed [DETONATION]"
			ShadowSeer_RegisterSpellCast(target, "Seed [DETONATION]")
		elseif (ShadowSeer_IsDelayedDamageSpell(spell)) then
			ShadowSeer_RegisterSpellCast(target, spell) -- for resistances
		end
			
		ShadowSeer_RegisterCritDamage(target, amount, UnitName("player"), spell, partialresist)
		return
	end

	start, iend, spell, target, amount = string.find(text, "Your (.+) hits (.+) for (.+) Shadow damage%.")
	if (not start) then
		start, iend, spell, target, amount = string.find(text, "Your (.+) hits (.+) for (.+) Fire damage%.")
	end

	if (start) then
		ShadowSeer_Log(arg1, target, amount, UnitName("player"), spell)
		-- special case: soC & SB
		if (spell == "Seed of Corruption") then
			spell = "Seed [DETONATION]"
			ShadowSeer_RegisterSpellCast(target, "Seed [DETONATION]")
		elseif (ShadowSeer_IsDelayedDamageSpell(spell)) then
			ShadowSeer_RegisterSpellCast(target, spell) -- for resistances
		end

		ShadowSeer_RegisterSpellDamage(target, amount, UnitName("player"), spell, partialresist)
		return
	end
		
	start, iend, spell, target = string.find(text, "Your (.+) was resisted by (.+)%.")
	if (start) then
		ShadowSeer_Log(arg1, target, 0, UnitName("player"), spell)

		-- special case: soC & SB
		if (spell == "Seed of Corruption") then
			ShadowSeer_RegisterSpellCast(target, "Seed [DETONATION]")
			ShadowSeer_RegisterResist(target, "Seed [DETONATION]")
		elseif (ShadowSeer_IsDelayedDamageSpell(spell)) then
			ShadowSeer_RegisterSpellCast(target, spell) -- for resistances
			ShadowSeer_RegisterResist(target, spell)
		else
			ShadowSeer_RegisterResist(target, spell)
		end
		return
	end
end

function ShadowSeer_ParseOtherDamage()
-- (caster)'s (spell) hit|crits (target) for (x) damage. (y resisted)
-- (caster)'s (spell) hit|crits (target) for (x) damage. 
	local start, iend, resist, amount, school, spell
	local text = arg1
		
	start, iend, partialresist = strfind(text,"%((.+) resisted%)")
	if (start) then
		text = string.sub(text, 1, string.find(text, "%."))
	else
		partialresist = 0
	end -- not using resist value yet for bookkeeping purposes, but might want to add this later on

	start, iend, caster, spell, target, amount = string.find(text, "(.+)'s (.+) crits (.+) for (.+) Shadow damage%.")
	if (start) then
		caster = string.sub(caster, 1, string.len(caster) - 1) -- I have no clue why, but it adds a space
		ShadowSeer_Log(arg1, target, amount, caster, spell)
		ShadowSeer_RegisterCritDamage(target, amount, caster, spell, partialresist)
		return
	end
	start, iend, caster, spell, target, amount = string.find(text, "(.+)'s (.+) hits (.+) for (.+) Shadow damage%.")
	if (start) then
		caster = string.sub(caster, 1, string.len(caster) - 1) -- I have no clue why, but it adds a space
		ShadowSeer_Log(arg1, target, amount, caster, spell)
		ShadowSeer_RegisterSpellDamage(target, amount, caster, spell, partialresist)
	end
	
	-- we don't monitor other people's fire damage
end

--
-- REGISTERING DATA
--
function ShadowSeer_RegisterDamage(plist, target, amount, player, spell, crit, partialresist)
	-- check if source is in raid
	if (ShadowSeer_GetClassForGroupMember(player) == nil and player ~= "Unknown") then
		return -- not monitoring damage from nonraid people
	end
	
	-- register damage in our level-based resulttable
	if (player == UnitName("player") and not string.find(spell, "%[DOT%]")) then
		ShadowSeer_RegisterOwnDamage(plist, target, spell, amount)
	end
	
	-- add to raidwide damage
	ShadowSeer_UpdateDamageList(plist[1].damagelist, target, amount, player, spell, crit, partialresist)

	-- add to player damage
	if (ShadowSeer_GetPlayer(plist, player)) then
		ShadowSeer_UpdateDamageList(ShadowSeer_GetPlayer(plist, player).damagelist, target, amount, player, spell, crit, partialresist);
	else
		if (player == "Unknown") then  -- not in list, add player
			ShadowSeer_CreatePlayer(plist, player, "Paladin")
		else
			ShadowSeer_CreatePlayer(plist, player, ShadowSeer_GetClassForGroupMember(player))
		end
		ShadowSeer_UpdateDamageList(plist[table.getn(plist)].damagelist, target, amount, player, spell, crit, partialresist)		
	end
	
	-- add ISB contrib
	local caster = ShadowSeer_TargetIsDebuffed(target)
	if (caster and ShadowSeer_GetSchoolForSpell(spell) == "Shadow") then
		plist[1].ISBcontribution = plist[1].ISBcontribution + amount
		local cid = ShadowSeer_GetPlayer(plist, caster)
		if (cid) then
			cid.ISBcontribution = cid.ISBcontribution + amount
		else
			ShadowSeer_Log("Error: " .. caster .. "'s ISB contribution couldn't be added to")
		end
	end
end
	
function ShadowSeer_UpdateDamageList(dl, target, amount, player, spell, crit, partialresist)
	ShadowSeer_UpdateDamageField(dl[1], target, amount, player, spell, crit, partialresist)
	for i=2, table.getn(dl) do
		if(dl[i].spellname == spell) then
			-- if exists, update field
			ShadowSeer_UpdateDamageField(dl[i], target, amount, player, spell, crit, partialresist)
			return
		end
	end

	tinsert(dl, ShadowSeer_CreateDamageList(spell))
	ShadowSeer_UpdateDamageField(dl[table.getn(dl)], target, amount, player, spell, crit, partialresist)
end

function ShadowSeer_UpdateDamageField(sf, target, amount, player, spell, crit, partialresist)
	if (crit) then
		sf.critcount = sf.critcount + 1
		sf.totalcritdamage = sf.totalcritdamage + amount
	end
	sf.ticcount = sf.ticcount + 1
	sf.totaldamage = sf.totaldamage + amount
	sf.maxdamage = max(sf.maxdamage, amount)
	sf.partialresist = sf.partialresist + partialresist
	if ShadowSeer_TargetIsDebuffed(target) then
		sf.ampeddamage = sf.ampeddamage + amount
	end
end

function ShadowSeer_RegisterDoTDamage(target, amount, player, spell, partialresist)
	ShadowSeer_RegisterDamage(ShadowSeer_PlayerList_AllCombats, target, amount, player, spell .. " [DOT]", false, partialresist)
	ShadowSeer_RegisterDamage(ShadowSeer_PlayerList_LastCombat, target, amount, player, spell .. " [DOT]", false, partialresist)
	ShadowSeer_RedoBars()
end

function ShadowSeer_RegisterCritDamage(target, amount, player, spell, partialresist)
	if (ShadowSeer_GetClassForGroupMember(player) == nil and player ~= "Unknown") then
		return -- not monitoring damage from nonraid people
	end

	ShadowSeer_RegisterDamage(ShadowSeer_PlayerList_AllCombats, target, amount, player, spell, true, partialresist)
	ShadowSeer_RegisterDamage(ShadowSeer_PlayerList_LastCombat, target, amount, player, spell, true, partialresist)

	-- exception, shadowbolt crits cause debuffs
	if (spell == "Shadow Bolt" and ShadowSeer_WarlockHasISB(player) and player ~= "Unknown") then
		ShadowSeer_AddToCritList(target, player)
		ShadowSeer_ChangeDebuffOwner(target, player)
		ShadowSeer_PlayerList_AllCombats[1].SBcrits = ShadowSeer_PlayerList_AllCombats[1].SBcrits + 1
		ShadowSeer_PlayerList_LastCombat[1].SBcrits = ShadowSeer_PlayerList_LastCombat[1].SBcrits + 1
		ShadowSeer_GetPlayer(ShadowSeer_PlayerList_AllCombats, player).SBcrits = ShadowSeer_GetPlayer(ShadowSeer_PlayerList_AllCombats, player).SBcrits + 1
		ShadowSeer_GetPlayer(ShadowSeer_PlayerList_LastCombat, player).SBcrits = ShadowSeer_GetPlayer(ShadowSeer_PlayerList_LastCombat, player).SBcrits + 1
	end

	if (player == UnitName("player")) then -- register own crits on the spell result table
		sf = ShadowSeer_FindSpellField(ShadowSeer_PlayerList_AllCombats, target, spell)
		sf.critcount = sf.critcount + 1
		sf = ShadowSeer_FindSpellField(ShadowSeer_PlayerList_LastCombat, target, spell)
		sf.critcount = sf.critcount + 1
	end
	
	ShadowSeer_RedoBars()
end

function ShadowSeer_RegisterSpellDamage(target, amount, player, spell, partialresist)
	ShadowSeer_RegisterDamage(ShadowSeer_PlayerList_AllCombats, target, amount, player, spell, false, partialresist)
	ShadowSeer_RegisterDamage(ShadowSeer_PlayerList_LastCombat, target, amount, player, spell, false, partialresist)
	ShadowSeer_RedoBars()
end


--
-- utility
--
function ShadowSeer_GetClassForGroupMember(pname) 
	-- might be me
	if (pname == UnitName("player")) then
		return UnitClass("player")
	end

	-- might be in PARTY
	for i=1,4 do 
		if (pname == UnitName("party" .. i)) then
			return UnitClass("party" .. i)
		end
	end

	local name, rank, subgroup, level, class, fileName, zone, online, isDead
	local numRaidMembers = GetNumRaidMembers()
	for i=1, numRaidMembers, 1 do
		name, rank, subgroup, level, class, fileName, zone, online, isDead = GetRaidRosterInfo(i)
		if (name == pname) then
			return class
		end
	end

	return nil 
end



