
-- 
-- TEMP BUFF LOGGING
--

-- scans current buffs for ShadowSeer_BuffScanNames and adds it to the list if it's found and not filtered
function ShadowSeer_ScanBuffs()
	-- figure out which buff it was
	local buffid = -1
	for id = 1,24 do
		local name, rank = GetPlayerBuffName(id)
		local b = 1
		if (name) then
			while (b <= table.getn(ShadowSeer_BuffsToScan)) do
				if (name == ShadowSeer_BuffsToScan[b].name) then
					if (name == "Aura of the Crusader") then
						local stackcount = GetPlayerBuffApplications(id)
						ShadowSeer_RegisterBuff(id, "Aura of the Crusader", ShadowSeer_BuffsToScan[b].received, stackcount)
						table.remove(ShadowSeer_BuffsToScan, b)
					else
						ShadowSeer_RegisterBuff(id, name, ShadowSeer_BuffsToScan[b].received)
						table.remove(ShadowSeer_BuffsToScan, b)
					end
				else
					b = b + 1
				end
			end
		end
	end
end
	
-- add a given buff to the list, if it passes the filter
function ShadowSeer_RegisterBuff(buffid, bname, received, stacks)
	if (GetTime() - received > 5) then
		ShadowSeer_Log("Buff " .. bname .. " has timed out. Received " .. GetTime() - received .. "s ago.")
		return -- timed out 
	end

	if (bname == "Ferocious Inspiration" or bname == "Lifebloom" or bname == "Brilliance Aura") then
		return
	end
	
	-- buff found, set up my invisible tooltip with the buff info
	local buffIndex, untilCancelled = GetPlayerBuff(buffid, "HELPFUL");
	
	MyScanningTooltip:ClearLines()
	MyScanningTooltip:SetPlayerBuff(buffIndex)
	local bufftext = getglobal("MyScanningTooltipTextLeft2"):GetText()

	if (not bufftext) then
		ShadowSeer_Log("Error: couldn't find data for " .. bname)
		return -- something went screwy. abort
	end
	bufftext = string.lower(bufftext)
	
	if (not string.find(bufftext, "damage") or not string.find(bufftext, "by") or string.find(bufftext, "%%")) then
		return -- fel armor ignored through this, but we don't particularly care about it
	end

	local school = "All"
	if (string.find(bufftext, "shadow")) then
		if (string.find(bufftext, "fire")) then
			school = "All"
		else
			school = "Shadow"
		end
	elseif (string.find(bufftext, "fire")) then
		school = "Fire"
	elseif (string.find(bufftext, "nature") or string.find(bufftext, "arcane") or string.find(bufftext, "frost") or string.find(bufftext, "holy") or string.find(bufftext, "physical")) then
		ShadowSeer_Log("Buff " .. bname .. " rejected, wrong school.")
		return
	end
		
	if (string.find(bufftext, "damage taken") or string.find(bufftext, "damage received")) then
		return -- amp magic
	end

	if (string.find(bufftext, "attacker")) then
		return -- fire shield / thorns, etc
	end
	
	local st, en, amount = string.find(bufftext, "(%d+)")
	if (not amount) then
		return
	end
	if (bname == "Aura of the Crusader") then
		amount = 8
	end
	
	if (GetPlayerBuffTimeLeft(buffid) > 35) then -- filter out longer buffs
		return
	end

	ShadowSeer_Log("Buff Detected: " .. bname, amount, received)

	ShadowSeer_UpdateBuffList(ShadowSeer_GetPlayer(ShadowSeer_PlayerList_AllCombats, UnitName("player")).proctable , bname, amount, buffid, received, stacks, school)
	ShadowSeer_UpdateBuffList(ShadowSeer_GetPlayer(ShadowSeer_PlayerList_LastCombat, UnitName("player")).proctable , bname, amount, buffid, received, stacks, school)
end	

function ShadowSeer_UpdateBuffList(bufflist, bname, amount, buffid, received, stacks, school)
	for i=1,getn(bufflist) do
		local b = bufflist[i]
		if (b.name == bname) then
			if (GetTime() - b.lastprocced < b.cooldown or b.cooldown == -1) then
				b.cooldown = GetTime() - b.lastprocced
			end
			b.lastprocced = GetTime()
			b.expires = GetPlayerBuffTimeLeft(buffid) + GetTime()
			b.proccount = b.proccount + 1
			b.active = true
			if (stacks) then
				b.stacks = stacks
			end
			return
		end
	end

	local b_rec = {}
	b_rec["name"] = bname
	b_rec["spellpower"] = amount
	b_rec["lastprocced"] = GetTime()
	b_rec["expires"] = GetPlayerBuffTimeLeft(buffid) + GetTime() 
	b_rec["duration"] = GetPlayerBuffTimeLeft(buffid) + (GetTime() - received) -- estimated full length for reporting
	b_rec["proccount"] = 1
	b_rec["cooldown"] = -1
	b_rec["spellcount"] = 0
	b_rec["stackcount"] = 0
	b_rec["school"] = school
	b_rec["stacks"] = 1
	b_rec["active"] = true
	table.insert(bufflist, b_rec)
end 
	
function ShadowSeer_DeactivateBuff(name)
	ShadowSeer_DeactivateBuffInList(ShadowSeer_GetPlayer(ShadowSeer_PlayerList_AllCombats, UnitName("player")).proctable, name)
	ShadowSeer_DeactivateBuffInList(ShadowSeer_GetPlayer(ShadowSeer_PlayerList_LastCombat, UnitName("player")).proctable, name)
	ShadowSeer_Log(name .. " faded.")
end

function ShadowSeer_DeactivateBuffInList(btable, name)
	for i=1,getn(btable) do
		local b = btable[i]
		if (b.name == name) then
			b.active = false
			return
		end
	end
end

-- returns a table with names of all known temporary buffs active on the player
function ShadowSeer_GetActiveBuffs()
	local blist = {}
	local allbuffs = ShadowSeer_GetPlayer(ShadowSeer_PlayerList_AllCombats, UnitName("player")).proctable
	
	for i=1,getn(allbuffs) do
		local b = allbuffs[i]
		if (b.active) then
			if (ShadowSeer_TimeCastStarted < b.expires + 3) then -- 3 second sanity check. If the buff didn't last this long , we must have missed it fading (due to player death or unforeseen stuff)
				table.insert(blist, b.name)
			else
				b.active = false
			end
		end
	end
	return blist
end

function ShadowSeer_RegisterSpellCastInTempBuffList(sname, blist)
	ShadowSeer_RegisterSpellCastInTempBuffListWithPlayerList(ShadowSeer_PlayerList_AllCombats, sname, blist)
	ShadowSeer_RegisterSpellCastInTempBuffListWithPlayerList(ShadowSeer_PlayerList_LastCombat, sname, blist)
end

function ShadowSeer_RegisterSpellCastInTempBuffListWithPlayerList(plist, sname, blist)
	local school = ShadowSeer_GetSchoolForSpell(sname)
	local proclist = ShadowSeer_GetPlayer(plist, UnitName("player")).proctable
	for i=1,table.getn(blist) do
		for j=1,table.getn(proclist) do
			if (blist[i] == proclist[j].name and (proclist[j].school == "All" or proclist[j].school == school)) then
				proclist[j].spellcount = proclist[j].spellcount + 1
				proclist[j].stackcount = proclist[j].stackcount + proclist[j].stacks
			end
		end
	end
end

function ShadowSeer_IsWarlockCombatSpell(spellname)
-- DD
	if (spellname == "Shadow Bolt") then return true
	elseif (spellname == "Shadowburn") then return true
	elseif (spellname == "Shadowfury") then return true
	elseif (spellname == "Death Coil") then return true

-- channeled
	elseif (spellname == "Drain Life") then return true
	elseif (spellname == "Drain Soul") then return true
	elseif (spellname == "Drain Mana") then return true

-- curses
-- (filtering those that don't gain from spellpower
	elseif (spellname == "Curse of Agony") then return true
	elseif (spellname == "Curse of Shadow") then return true
	elseif (spellname == "Curse of the Elements") then return true
	elseif (spellname == "Curse of Doom") then return true
	elseif (spellname == "Curse of Exhaustion") then return true
	elseif (spellname == "Curse of Recklessness") then return true
	elseif (spellname == "Curse of Tongues") then return true
	elseif (spellname == "Curse of Weakness") then return true

-- dots
	elseif (spellname == "Seed of Corruption") then return true
	elseif (spellname == "Corruption") then return true
	elseif (spellname == "Unstable Affliction") then return true
	elseif (spellname == "Siphon Life") then return true

-- support 
	elseif (spellname == "Dark Pact") then return true
	elseif (spellname == "Life Tap") then return true
--	elseif (spellname == "Soulshatter") then return true

-- fire spells
	elseif (spellname == "Immolate") then return true
	elseif (spellname == "Incinerate") then return true
	elseif (spellname == "Searing Pain") then return true
	elseif (spellname == "Soul Fire") then return true
	elseif (spellname == "Conflagrate") then return true
	elseif (spellname == "Rain of Fire") then return true
	elseif (spellname == "Hellfire") then return true
	
	else return false
	end
end	

function ShadowSeer_IsSpellBoostedBySpellPower(spellname)
-- DD
	if (spellname == "Shadow Bolt") then return true
	elseif (spellname == "Shadowburn") then return true
	elseif (spellname == "Shadowfury") then return true
	elseif (spellname == "Death Coil") then return true

-- channeled
	elseif (spellname == "Drain Life") then return true
	elseif (spellname == "Drain Soul") then return true

-- curses
	elseif (spellname == "Curse of Agony") then return true
	elseif (spellname == "Curse of Doom") then return true

-- dots
	elseif (spellname == "Seed of Corruption") then return true
	elseif (spellname == "Corruption") then return true
	elseif (spellname == "Unstable Affliction") then return true
	elseif (spellname == "Siphon Life") then return true

-- support 
	elseif (spellname == "Dark Pact") then return true
	elseif (spellname == "Life Tap") then return true

-- fire spells
	elseif (spellname == "Immolate") then return true
	elseif (spellname == "Incinerate") then return true
	elseif (spellname == "Searing Pain") then return true
	elseif (spellname == "Soul Fire") then return true
	elseif (spellname == "Conflagrate") then return true
	elseif (spellname == "Rain of Fire") then return true
	elseif (spellname == "Hellfire") then return true
	else return false
	end
end
